/**********************************************************************************************************
This file proccesses the main test variables. 

PROCEDURES:
	- Step 1. Aggregate state-level earnings measures.
	- Step 2. Calculate control variables.
	- Step 3. Calculate state-bond returns.
	- Step 4. Merge datasets.
	- Step 5. Export data.

**********************************************************************************************************/

/**********************************************************************************************************
****************STEP 1. Aggregate State-level (Seasonal-adjusted) Earnings Measure*************************
**********************************************************************************************************/

/* Take firms' seasonal lags*/
data comp; set Data.comp_2000_2017; data_qtr=qtr(datadate); data_year=year(datadate); run; 
proc sort; by gvkey1 data_qtr data_year; run;
data comp; set comp;by gvkey1 data_qtr; 
	format calendar_mon_end yymmdd10.;
	calendar_mon_end=intnx('month',rdq, 0, 'e');
	year=year(calendar_mon_end);

	piq_ss_lag=lag(piq); mkvaltq_ss_lag=lag(mkvaltq);

	if first.data_qtr then do;
			piq_ss_lag=.; mkvaltq_ss_lag=.;	end;  /*set quarters in first year to "."*/
run; 

/* Aggregate by (historical headquartered) state & reporting (rdq) month */
proc means data= comp noprint nway;
	class state calendar_mon_end; 	var  piq piq_ss_lag mkvaltq_ss_lag equity_ret; 
	output out=GeoFa 				sum= piq piq_ss_lag mkvaltq_ss_lag equity_ret;  
run;

/* Calculate earnings signals*/
data GeoFa; set GeoFa;
	if 2005<=year(calendar_mon_end)<=2017;
	format calendar_qtr_end mmddyy10.;
	calendar_qtr_end=intnx('quarter',calendar_mon_end, 0, 'e');
	month=month(calendar_mon_end);

	piq_q1=(piq-piq_ss_lag)/mkvaltq_ss_lag;
	equity_ret1=equity_ret/_freq_; equity_ret=equity_ret1; drop equity_ret1; 
	keep state calendar_qtr_end calendar_mon_end piq_q1 equity_ret;
run;
proc sort; by state calendar_mon_end; run;

/* Set final aggregate earnings signals */
Data Data.Geofa; set GeoFa; run;



/**********************************************************************************************************
*******************************Step 2. Calculate Control variables*****************************************
**********************************************************************************************************/

/*******************************A. Gross State Product Growth*********************************************/
proc import out=ctrl_gsp datafile="controls" 
	dbms= xlsx replace; sheet="gsp"; getnames=yes;	run;
proc sort; by state calendar_qtr_end; run;
data ctrl_gsp; set ctrl_gsp; by state;
	gspg=gsp/lag(gsp)-1;
	if first.state then do; 
		gspg=.; end;
run;
Data Data.ctrl_gsp; set ctrl_gsp; run;


/*************************************B. Unemployment Rate Growth*****************************************/
proc import out=ctrl_ur datafile="controls" 
	dbms= xlsx replace; sheet="ur"; getnames=yes;	run;
data ctrl_ur(drop=_label_ _NAME_); set ctrl_ur;
	date1=date-1;
	state= substr(_name_, 1, 2);
 	format calendar_qtr_end YYMMDD10. date1 YYMMDD10.;
	calendar_qtr_end = INTNX('quarter', date1, 0, 'e' );
run;
proc sort nodupkey; by state calendar_qtr_end date1; run;
data ctrl_ur; set ctrl_ur; by state calendar_qtr_end;
 	if last.calendar_qtr_end;
run;
proc sort nodupkey; by state calendar_qtr_end; run;
data ctrl_ur; set ctrl_ur; by state;
	urg=ur/lag(ur)-1;	
	if first.state then urg=.;
run;
Data Data.ctrl_ur; set ctrl_ur; run;


/**************************************C. Coincident Index Growth*****************************************/
proc import out=ctrl_coin datafile="controls" 
	dbms= xlsx replace; sheet="coin"; getnames=yes;	run;
data ctrl_coin; set ctrl_coin;
	date1=date-1;
	calendar_qtr_end = INTNX('quarter', date1, 0, 'e' );
	format date1 YYMMDD10. date YYMMDD10. calendar_qtr_end YYMMDD10.;
run;
proc transpose data=ctrl_coin
	out=ctrl_coin2(rename=(col1=coin));
  	var AL AK AZ AR CA CO CT DE FL GA HI ID IL IN IA KS KY LA
	    ME MD MA MI MN MS MO MT NE NV NH NJ NM NY NC ND OH OK 
	    OR PA RI SC SD TN TX UT VT VA WA WI WV WY;
   	by date1 calendar_qtr_end;
run;
data ctrl_coin2 (drop=_NAME_); set ctrl_coin2 (rename=(_LABEL_=state)); run;
proc sort; by state calendar_qtr_end date1; run;
data ctrl_coin2; set ctrl_coin2; by state calendar_qtr_end;
	if last.calendar_qtr_end;
run;
proc sort; by state calendar_qtr_end; run;
data ctrl_coin2; set ctrl_coin2; by state;
	coing=(coin/lag(coin))-1;
	if first.state then coing=.;
run;
Data Data.ctrl_coin; set ctrl_coin2; run;


/*****************************************D. S&P Credit Ratings*******************************************/
proc import out=rating datafile="controls" 
	dbms= xlsx replace; sheet="rating"; getnames=yes; run;
data rating; set rating;
	if rating='AAA' then sp_rtg= 23; 
	if rating='AA+'	then sp_rtg= 22;
	if rating='AA' 	then sp_rtg= 21;
	if rating='AA-' then sp_rtg= 20;
	if rating='A+' 	then sp_rtg= 19;
	if rating='A' 	then sp_rtg= 18;
	if rating='A-' 	then sp_rtg= 17;
	if rating='BBB+' 	then sp_rtg= 16;
	if rating='BBB' 	then sp_rtg= 15;
	if rating='BBB-' 	then sp_rtg= 14;
	if rating='BB+' 	then sp_rtg= 13;
	if rating='BB' 		then sp_rtg= 12;
	if rating='BB-' 	then sp_rtg= 11;
	if rating='B+' 		then sp_rtg= 10;
	if rating='B' 		then sp_rtg= 9;
	if rating='B-' 		then sp_rtg= 8;
	if rating='CCC+' 	then sp_rtg= 7;
	if rating='CCC' 	then sp_rtg= 6;
	if rating='CCC-' 	then sp_rtg= 5;
	if rating='CC' 		then sp_rtg= 4;
	if rating='C' 		then sp_rtg= 3;
	if rating='DDD' 	then sp_rtg= 2;
	if rating='DD' 		then sp_rtg= 1;
	if rating='D' 		then sp_rtg= 0;
	rename state=state_name; rename code=state_code;
run;
proc sort; by state_code year; run;
data rating; set rating; by state_code;
	dsp_rtg=sp_rtg-lag(sp_rtg);
	if first.state_code then dsp_rtg=.;
run;
Data Data.ctrl_rating; set rating; run;

/***************************************E. State Government "GENERAL FUND" Data **************************/
proc import out=ctrl_GvtFa_gen datafile="controls" 
	dbms= xlsx replace; sheet="gvtfa_gen"; getnames=yes; run;
proc sort; by state year; run;
data ctrl_GvtFa_gen; set ctrl_GvtFa_gen; by state;
	format fyend yymmdd10.; fyend=input(trim(Fiscal_Year_End), yymmdd8.);
	lat_gen=log(Total_Assets);
	current_gen=(Cash___Near_Cash+Accounts___Notes_Receivable)/(Accounts_Payable+Accrued_Liabilities); 
	d_current_gen=current_gen-lag(current_gen);
	if first.state then do; d_current_gen=.; end;
	keep government_type state year fyend lat_gen d_current_gen;
run;
Data Data.ctrl_GvtFa_gen; set ctrl_GvtFa_gen; run;


/********************************F. States Government "GOVERNMENT FUND" Data ******************************/
proc import out=ctrl_GvtFa_govt datafile="controls" 
	dbms= xlsx replace; sheet="GvtFa_govt"; getnames=yes; run;
proc sort; by state year; run;
data ctrl_GvtFa_govt; set ctrl_GvtFa_govt; by state;
	format fyend yymmdd10.; fyend=input(trim(Fiscal_Year_End), yymmdd8.);
	dnetasset_percap=change_in_net_assets_per_capita*1;
	lat=log(Total_Assets);
	current=Current_Assets_Reported/Current_Liabilities; 
	d_current=current-lag(current);
	if first.state then do; d_current=.; end;
	keep government_type state year fyend lat d_current dnetasset_percap;
run;
Data Data.ctrl_GvtFa_govt; set ctrl_GvtFa_govt; run;

/**********************************************************************************************************
***********************************STEP 3. Calculate State-bond Returns************************************
**********************************************************************************************************/

/*****************************I. Import SDC cusip and Bloomberg data**************************************/
/* Prepare MSRB data */
data msrb; set Data.msrb_2005_2017;
	cusip6=substr(cusip, 1,6); 
run;

/* Import SDC CUSIP list (uniquely identified by cusip6)*/
proc import out=sdc_cusip_state datafile="sdc_cusip_list" 
	dbms= xlsx replace; sheet="state"; getnames=yes; run;

/* Merge MRSB cusip with SDC cusip list*/
proc sql; create table msrb_sdc_state
	as select a.*, b.issuer as sdc_issuer, b.state as sdc_state
    from msrb as a inner join sdc_cusip_state as b
    on (a.cusip6=b.cusip6); quit;

/* Import and prepare Bloomberg bond characterstics**/
proc import out=bloomberg datafile="Bloomberg" 
	dbms= xlsx replace; sheet="state"; getnames=yes; run;
data bloomberg; set bloomberg; 
	if MUNI_ISSUE_TYP='G.O. UNLTD NOTES' or MUNI_ISSUE_TYP='GENERAL OBLIGATION LTD' or MUNI_ISSUE_TYP='GENERAL OBLIGATION UNLTD' then go_bond=1; else go_bond=0;
	if SECURITY_TYP='FIXED' or SECURITY_TYP='FIXED, OID' or SECURITY_TYP='ZERO COUPON' or SECURITY_TYP='ZERO COUPON, OID' then fixed_bond=1; else fixed_bond=0;

	/*convert and format variables*/
	cpn1=cpn*1;		yield_on_issue_date1=yield_on_issue_date*1;		par_amt1=par_amt*1; 	issue_px1=issue_px*1;
	muni_mty_size1=muni_mty_size*1;		muni_issue_size1=muni_issue_size*1;
	format issue_dt1 mmddyy10. first_settle_dt1 mmddyy10. maturity1 mmddyy10.;
	issue_dt1=input(trim(issue_dt),ddmmyy10.);	first_settle_dt1=input(trim(first_settle_dt),ddmmyy10.);	maturity1=input(trim(maturity),ddmmyy10.); 
	
	/*keep only fixed-rate GO bonds*/
	if fixed_bond=1; 
	if go_bond=1;
run;


/***********************************II. Combine MSRB and Bloomberg ***************************************/
/* Merge Bloomberg with MSRB data */
proc sort data=msrb_sdc_state; by cusip; run;
proc sql; create table msrb_bloomberg
	as select 	a.*, b.issuer
from msrb_sdc_state as a inner join bloomberg as b on (a.cusip=b.cusip); 
quit;

/* Prepare MSRB data*/
data msrb_sdc_state1; set msrb_sdc_state; where dollar_price>1;

	par_traded1=par_traded*1;
	dollar_price_w=dollar_price_dirty*par_traded1;

	/*specify trade type - exclude inter-dealer trades*/
	if TRADE_TYPE_INDICATOR='P' or TRADE_TYPE_INDICATOR='S';

run;


/*******************************III. Create Monthly Return *******************************************/

/* Take weighted average of prices on each trading date per cusip*/
proc means data=msrb_sdc_state1 noprint nway;
	class cusip trade_date; var dollar_price_dirty dollar_price_w par_traded1;
	output out=return sum= mean=/autoname; run;

/* Keep one closest date to the month-end or month-beiginning dates*/
data return(drop=_type_ _freq_); set return;

	/*create month ends and beginnings*/
	format last_mon_end yymmdd10. next_mon_end yymmdd10. return_mon_end yymmdd10.;
    last_mon_end=intnx('month',trade_date, -1, 'e');
    next_mon_end=intnx('month',trade_date, 0, 'e');
 
	daystolast=trade_date-last_mon_end;		 /*days from month beginning to trade*/
	daystonext=next_mon_end-trade_date;		 /*days from month end to trade*/

	if 0<=daystolast<5 	then return_mon_end=last_mon_end; else daystolast=.;
	if 0<=daystonext<5 	then return_mon_end=next_mon_end; else daystonext=.;

	if missing(return_mon_end) then delete;		
run;

data prc1; set return; where not missing(daystolast); run;
proc sort; by cusip return_mon_end daystolast; run;
data prc1 (keep= cusip return_mon_end tradedate1 par_trade_mean1 prc_avg1 prc_vw1 par_trade_sum1 daystolast); 
	set prc1; by cusip return_mon_end;
	if first.return_mon_end;
	rename TRADE_DATE=tradedate1;
	rename par_traded1_Mean=par_trade_mean1;
	rename par_traded1_sum=par_trade_sum1;
	rename dollar_price_dirty_Mean=prc_avg1;
	rename dollar_price_w_Sum=prc_vw1;
run;

data prc2; set return; where not missing(daystonext); run;
proc sort; by cusip return_mon_end daystonext; run;
data prc2 (keep=cusip return_mon_end tradedate2 par_trade_mean2 prc_avg2 prc_vw2 par_trade_sum2 daystonext); 
	set prc2; by cusip return_mon_end;
	if first.return_mon_end;
	rename TRADE_DATE=tradedate2;
	rename par_traded1_Mean=par_trade_mean2;
	rename par_traded1_sum=par_trade_sum2;
	rename dollar_price_dirty_Mean=prc_avg2;
	rename dollar_price_w_Sum=prc_vw2;
run;

data return (keep=cusip return_mon_end tradedate prc_avg prc_vw par_trade_mean cusip6); merge prc1 prc2; by cusip return_mon_end;
	format tradedate yymmdd10.;
	cusip6=substr(cusip, 1,6);

	if missing(tradedate1)=1 and missing(tradedate2)=0 then do; 
		prc_vw=prc_vw2/par_trade_sum2; prc_avg=prc_avg2; par_trade_mean=par_trade_mean2; tradedate=tradedate2; end;
	if missing(tradedate1)=0 and missing(tradedate2)=1 then do; 
		prc_vw=prc_vw1/par_trade_sum1; prc_avg=prc_avg1; par_trade_mean=par_trade_mean1; tradedate=tradedate1; end;

	if missing(tradedate1)=0 and missing(tradedate2)=0 and daystolast<daystonext then do; 
		prc_vw=prc_vw1/par_trade_sum1; prc_avg=prc_avg1;par_trade_mean=par_trade_mean1; tradedate=tradedate1; end;
	if missing(tradedate1)=0 and missing(tradedate2)=0 and daystolast>daystonext then do; 
		prc_vw=prc_vw2/par_trade_sum2; prc_avg=prc_avg2; par_trade_mean=par_trade_mean2; tradedate=tradedate2; end;

	if missing(tradedate1)=0 and missing(tradedate2)=0 and daystolast=daystonext then do; 
		prc_vw=(prc_vw1+prc_vw2)/(par_trade_sum1+par_trade_sum2); prc_avg=(prc_avg1+prc_avg2)/2; par_trade_mean=(par_trade_mean1+par_trade_mean2)/2; tradedate=(tradedate2+tradedate1)/2; end;

	if missing(prc_vw) then delete;
run;
proc sort nodupkey; by cusip return_mon_end; run; 

proc sql; create table return2
	as select 	a.*,
		b.prc_vw as prc_vw_begin, b.prc_avg as prc_avg_begin, b.tradedate as tradedate_begin, b.par_trade_mean as par_trade_mean_begin
    from return as a 
		inner join return as b on (a.cusip=b.cusip) and (intnx('month',a.return_mon_end, -1, 'e')=b.return_mon_end); quit;

/* Prepare treasury returns */
data tbill; set Data.treasury;
	format tbill_mon_end yymmdd10.;
    tbill_mon_end=intnx('month',CALDT, 0, 'e');
run;

proc sql; create table return3
	as select 	a.*, 
				b.issuer, b.go_bond, b.state_code,
				b.cpn1 as blg_cpn, b.yield_on_issue_date1 as yield_on_issue_date, b.par_amt1 as par_amt, b.issue_px1 as issue_px,
				b.muni_mty_size1 as muni_mty_size, b.muni_issue_size1 as muni_issue_size, 
				b.issue_dt1 as issue_dt, b.first_settle_dt1 as first_settle_dt, b.maturity1 as maturity_dt, b.cpn_month1, b.cpn_month2,
			  	c.B30RET, c.B20RET, c.B10RET, c.B7RET, c.B5RET, c.B2RET, c.B1RET, c.T90RET, c.T30RET, c.CPIRET,
				d.state as sdc_state, d.issuer as sdc_issuer
    from return2 as a 
				inner join bloomberg as b on (a.cusip=b.cusip)
				inner join tbill as c on (a.return_mon_end=c.tbill_mon_end)
				left join sdc_cusip_state as d on (a.cusip6=d.cusip6); 
quit;

/* Calculate returns, adjusted for treasury rates and coupon payments */
data Data.return; set return3; 
	year=year(return_mon_end);
	month=month(return_mon_end);

	/*set treasury monthly returns to match with the term of the munis*/
	nyear=intck('year',issue_dt,maturity_dt);
	if nyear<=1 		then tbill_ret=B1RET;
	if 2<=nyear<=3 		then tbill_ret=B2RET;
	if 4<=nyear<=5 		then tbill_ret=B5RET;
	if 6<=nyear<=7 		then tbill_ret=B7RET;
	if 8<=nyear<=14 	then tbill_ret=B10RET;
	if 15<=nyear<=24 	then tbill_ret=B20RET;
	if nyear>=25 		then tbill_ret=B30RET;
	if missing(nyear) 	then tbill_ret=0;

	/*define interval for prorating returns*/
	interval=tradedate-tradedate_begin;  /*days between beginning-of-month and end-of-month trades*/

	/*define coupon rates*/
	if month=cpn_month1 or month=cpn_month2  then cpn=blg_cpn/2; else cpn=0;

	/*calculate treasury-adjusted, buy-and-hold monthly returns*/
	ret_avg=(((prc_avg+cpn)/prc_avg_begin)**(1/interval))**(day(return_mon_end))-1-tbill_ret;
	ret_w=(((prc_vw +cpn)/prc_vw_begin)**(1/interval))**(day(return_mon_end))-1-tbill_ret;

	/*calculate average trade size*/
	size=(par_trade_mean+par_trade_mean_begin)/2;

	drop tradedate prc_vw prc_avg par_trade_mean prc_vw_begin prc_avg_begin tradedate_begin par_trade_mean_begin;
run;


/**********************************************************************************************************
****************************************STEP 5.  Merge all datasets****************************************
**********************************************************************************************************/

/* Merge MSRB with aggregate earnings signals */
proc sql; create table Geofa_return
	as select a.*, b.*
	from Data.return as a 
		left join Data.GeoFa as b on (a.sdc_state = b.state) and (a.return_mon_end=b.calendar_mon_end)
; quit;

/* Merge with control variables*/
proc sql; create table Geofa_return2
	as select a.*, 
			b.dnetasset_percap, b.lat, b.d_current, c.lat_gen, c.d_current_gen,
			d.gspg, e.urg, f.coing, 
			g.sp_rtg, g.dsp_rtg
	from Geofa_return as a 
			left join Data.ctrl_GvtFa_govt as b on (a.sdc_state = b.state) and (b.fyend+365<a.calendar_mon_end<b.fyend+365*2) /*allow reporting lag*/
			left join Data.Ctrl_GvtFa_gen as c on (a.sdc_state = c.state) and (c.fyend+365<a.calendar_mon_end<c.fyend+365*2)
			left join Data.Ctrl_gsp as d on (a.sdc_state=d.state) and (a.calendar_qtr_end=d.calendar_qtr_end)
			left join Data.Ctrl_ur as e on (a.sdc_state=e.state) and (a.calendar_qtr_end=e.calendar_qtr_end)
			left join Data.Ctrl_coin as f on (a.sdc_state=f.state) and (a.calendar_qtr_end=f.calendar_qtr_end)
			left join Data.Ctrl_rating as g on (a.sdc_state=g.state_code) and (a.year=g.year);
quit;

/* Set data requirement*/
data Geofa_return3; set Geofa_return2;

	/*require non-missing values*/
	if not missing(ret_w);
	if not missing(ret_avg);
	if not missing(piq_q1);
	if not missing(size);
	if not missing(sp_rtg);
	if not missing(lat);
	if not missing(d_current);
	if not missing(lat_gen);
	if not missing(d_current_gen);
	if not missing(dnetasset_percap);
	if not missing(gspg);
	if not missing(urg);
	if not missing(dsp_rtg);
	if not missing(equity_ret);

	/*set coincident index to 0 for uncovered regions*/
	if missing(coing) then coing=0;
run;

/* Require at least 2 trades per cusip */
proc means data=Geofa_return3 noprint nway;
	class state cusip; 		var ret_w;
	output out=stat1_cusip 	sum=ret_w; run;
proc sql; create table Geofa_return4
	as select a.*, b._FREQ_
		from Geofa_return3 as a 
			left join stat1_cusip as b on (a.cusip=b.cusip); quit;
data Geofa_return4; set Geofa_return4; 
	if _freq_>=2; drop _freq_; 
run;

Data Data.finaldata; set Geofa_return4; run;

/**********************************************************************************************************
**************************************STEP 5.  Export Dataset**********************************************
**********************************************************************************************************/
proc export data=Data.finaldata outfile= "data02152021.dta" replace; run;
